% This file computes TD for the model with capital

function [td, test1, test2] = compute_td_unified(P,T,shock,previousrun,nguess)

% aux
reltol = read_opt(P,'reltol',1e-5);
abstol = read_opt(P,'abstol',1e-5);
NMax   = read_opt(P,'NMax',1200);
options = bvpset('RelTol',reltol,'AbsTol',abstol,'NMax',NMax);
options = bvpset('RelTol',reltol,'AbsTol',abstol,'NMax',NMax,'Stats','on');


%%% 1. Set initial guess
% Steady state
[y_guess, ss] = steadystate_unified(P);

func = @(t0) steadystate_t_unified(P,t0);

P.n_y = length(y_guess);

% Set initial time grid
% a. if no time where we switch from exog to endog r:
t_guess = linspace(0,T,300);  
P.n_regions = 1;

% b. if there is a time, double the time to activate regions algorithm!
if 0 < P.T_exog_to_endog && P.T_exog_to_endog < T
    t_guess = [t_guess(t_guess<P.T_exog_to_endog) P.T_exog_to_endog P.T_exog_to_endog t_guess(t_guess>P.T_exog_to_endog)];  %;[linspace(0,P.T_exog_to_endog - 1e-5,30) P.T_exog_to_endog P.T_exog_to_endog linspace(P.T_exog_to_endog + 1e-5,T,30)];
    P.n_regions = P.n_regions + 1;
    
end

% c. is there also a time where we switch from endog to exog?
if P.T_exog_to_endog < P.T_endog_to_exog && P.T_endog_to_exog < T
    t_guess = [t_guess(t_guess<P.T_endog_to_exog) P.T_endog_to_exog P.T_endog_to_exog t_guess(t_guess>P.T_endog_to_exog)];  %;[linspace(0,P.T_exog_to_endog - 1e-5,30) P.T_exog_to_endog P.T_exog_to_endog linspace(P.T_exog_to_endog + 1e-5,T,30)];
    P.n_regions = P.n_regions + 1;
end

P.endog_r_at_T = (P.T_exog_to_endog < T & P.T_endog_to_exog >= T);
P.endog_r_at_0 = (P.T_exog_to_endog<=0 & P.T_endog_to_exog > 0);

if P.n_regions == 1, assert(P.endog_r_at_0 == P.endog_r_at_T); end


%%% CHECKS 
test1 = odefun(1e6, y_guess, P.endog_r_at_T, P, shock);
test2 = odefun(0, y_guess, P.endog_r_at_0, P, shock);
test3 = bcfun(repmat(y_guess,1,P.n_regions),repmat(y_guess,1,P.n_regions),P,shock);

% 2. Set entire path for guess
if nguess
    t_guess = linspace(0,T,nguess); %tradeoff between speed of bvp4c and speed of guess
    t_guess = [ linspace(0,T/2,nguess) T];
    solinit = bvpinit(t_guess,func); 
else
solinit = bvpinit(t_guess,y_guess);
    if exist('previousrun','var')
        if isstruct(previousrun)
            if isfield(previousrun,'yp')
                solinit = bvpinit(previousrun, [0, T]);
            else
            % use guess from previous run
            t_guess = previousrun.t;
            y_guess = previousrun.y_all(:,end);

            if t_guess(end-1) == t_guess(end)
                t_guess = t_guess(1:end-1);
            end
            solinit = bvpinit(t_guess,y_guess); 
            end
        end
    end
end 

% 3. Set the functions used for bvp4c
if P.n_regions > 1
    myodefun = @(t,y,region) odefun(t,y, mod((region+P.endog_r_at_0+1),2) ,P,shock);  % mod thing maps regions into when r is endog / exog
else
    myodefun = @(t,y) odefun(t,y,P.endog_r_at_T,P,shock);
end
mybcfun  = @(y0,yT) bcfun(y0,yT,P,shock);

tic;
sol = bvp4c(myodefun,...
            mybcfun,...
            solinit,...
            options);
toc;
td.sol = sol;

td.y_all = sol.y;
td.t = sol.x;i=1;
td.c_s = sol.y(i,:);i=i+1;
td.c_b = sol.y(i,:);i=i+1;
td.d = sol.y(i,:);i=i+1;
td.h_s = sol.y(i,:);i=i+1;
td.h_b = sol.y(i,:);i=i+1;
td.n = sol.y(i,:);i=i+1;
td.kappa = sol.y(i,:);i=i+1;

td.k = sol.y(i,:);i=i+1;
td.J = sol.y(i,:);i=i+1;
td.Q = sol.y(i,:);i=i+1;
td.q = sol.y(i,:);i=i+1;
td.p = sol.y(i,:);



for i = 1:length(td.t)
    t = td.t(i);
    endog_r = (t>=P.T_exog_to_endog & t<= P.T_endog_to_exog);
    [~, extra] = odefun(t,td.y_all(:,i),endog_r,P,shock);
    fields = fieldnames(extra);
    for l = 1:length(fields)
        td.(fields{l})(i) = extra.(fields{l});
    end
    td.d2i(i) = td.d(i) / extra.posttax_inc_b;
    td.debt_service_borrower(i) = td.r(i) * td.d2i(i);
    td.debt_service_GDP(i) = td.r(i) * td.d(i);
    td.B(i) = P.B(t);
    td.Ygap(i) = P.f(td.k(i),td.n(i),td.n(i),P.alpha,P.gamma(td.t(i)),P.land_share,P.sigma,P.kss) - 1;
end
td.net_worth_b = td.pledgable_wealth - td.d;
td.net_worth_s = td.a_s;
td.ss = ss;

% compute how much r jumps at any T breaks
if 0 < P.T_exog_to_endog && P.T_exog_to_endog < T
    t = P.T_exog_to_endog;
    [~, i] = min(abs(td.t - t));
    [~, extra_exog] = odefun(t,td.y_all(:,i),0,P,shock);
    [~, extra_endog] = odefun(t,td.y_all(:,i),1,P,shock);
    td.rgap_exog_to_endog = extra_endog.r - extra_exog.r;
end

if P.T_exog_to_endog < P.T_endog_to_exog && P.T_endog_to_exog < T
    t = P.T_endog_to_exog;
    [~, i] = min(abs(td.t - t));
    [~, extra_exog] = odefun(t,td.y_all(:,i),0,P,shock);
    [~, extra_endog] = odefun(t,td.y_all(:,i),1,P,shock);
    td.rgap_endog_to_exog = extra_exog.r - extra_endog.r;
end

end


%%% ODEFUN %%%
function [ydot, extra] = odefun(t,y,endog_r,P,shock)

% this function computes the derivative of the ode system
i=1;
c_s=y(i); i=i+1;
c_b=y(i); i=i+1;
d=y(i); i=i+1;
h_s=y(i); i=i+1;
h_b=y(i); i=i+1;
n=y(i); i=i+1; 
kappa=y(i); i=i+1;
k=y(i); i=i+1; 
J=y(i); i=i+1;
Q=y(i); i=i+1;
q=y(i); i=i+1;
p=y(i);


%%% Read out paths (we kicked all but the interest rate paths out of shock)
r_exog = shock.r_exog(t); r_exog_dot = shock.r_exog_dot(t);
g = P.g(t); g_dot = P.g_dot(t);
kappa_dot = P.kappa_dot(t);
tau_s = P.tau_s(t); tau_s_dot = P.tau_s_dot(t);
tau_b = P.tau_b(t); tau_b_dot = P.tau_b_dot(t);
r_ZLB = shock.r_ZLB(t);
exog_debt = P.exog_debt; exog_debt_d = shock.exog_debt(t); exog_debt_d_dot = shock.exog_debt(t);
B = P.B(t); B_dot = P.B_dot(t); B_ddot = P.B_ddot(t);

Yhat = P.f(k,n,n,P.alpha,P.gamma(0),P.land_share,P.sigma,P.kss);

% wage
jacobian = P.jacobian(k,n,n,P.alpha,P.gamma(t),P.land_share,P.sigma,P.kss);
hessian = P.hessian(k,n,n,P.alpha,P.gamma(t),P.land_share,P.sigma,P.kss);

% plug in value of derivatives for production function
f_t = jacobian(5) * P.gamma_dot(t); 
f_k = jacobian(1);
f_ns_k = hessian(2, 1);
f_nb_k = hessian(3, 1);
f_ns_ns = hessian(2, 2);
f_ns_nb = hessian(2, 3);
f_nb_ns = hessian(3, 2);
f_nb_nb = hessian(3, 3);
f_n = jacobian(2) + jacobian(3);

% wages
w_s = jacobian(2); 
w_b = jacobian(3);

% asset 
a_s = h_s + d + B + P.kappa_s * J;
a_b = h_b - d + P.kappa_b * J;

ZLB = 0;

% income
y_s = w_s * n; y_b = w_b * n;
omega_s = y_s / (y_s+y_b); omega_b = y_b / (y_s+y_b);

% tax shares
tax_share_s = P.chi * omega_s + P.phi(t);
tax_share_b = 1 - P.chi * omega_s - P.phi(t);

% firm profit, investment, firm value, and dividend
pi = Yhat - y_s - y_b;
MPK = jacobian(1);
I = P.epsilon * (Q-1) * k + P.delta * k;
Zeta = P.Zeta((I-P.delta*k)/k,P.epsilon);
Zeta_p = P.Zeta_p((I-P.delta*k)/k,P.epsilon);
div = pi - I - Zeta * k;

% Physical capital
k_dot = I - P.delta * k;

% potential output with full employment
Y_pot = P.f(k,1,1,P.alpha,P.gamma(t),P.land_share,P.sigma,P.kss);
jacobian_pot = P.jacobian(k,1,1,P.alpha,P.gamma(t),P.land_share,P.sigma,P.kss);
Y_pot_dot = jacobian_pot(1) * k_dot + jacobian_pot(5) * P.gamma_dot(t);

% relevant constants in the linear system to determine r

c_Zeta = k_dot * (Zeta - Zeta_p * I/k); 

c_I = P.epsilon*(Q-1)*k_dot + P.delta*k_dot;

c_Q = - MPK + Zeta - Zeta_p*I/k + Q*P.delta;

% Unconstrained Euler: r = c_rn - m * r
c_rn = (c_s * (P.rho_s + P.delta_s - P.delta_s*(c_s/P.rho_s/a_s)^P.inverse_eis * P.eta_s(a_s/Y_pot,P.a_thresh, P.eta0, P.eta1)) + ...
       c_b * (P.rho_b + P.delta_b - P.delta_b*(c_b/P.rho_b/a_b)^P.inverse_eis * P.eta_b(a_b/Y_pot, P.a_thresh, P.eta0, P.eta1, P.mu)) ...
      + P.inverse_eis *(Y_pot_dot - g_dot - (c_Zeta + (1 + Zeta_p) * (c_I + P.epsilon * k * c_Q)))) / (c_s + c_b);

m = P.inverse_eis * (1 + Zeta_p)*P.epsilon*k*Q /(c_s+c_b);

%%% Natural rate: Distinguish two cases: constrained & unconstrained
% Note that formally, the system of equations yields 
% r_n = c_rn - m min(r_n,r_constrained) (eq), c_rn and m constant (at t). r_n is unique since m > 0.
% For r that solves r = c_rn - m * r,
% either the system is unconstrained and r < r_constrained, (eq) holds for the accurate natural rate, or
% the system is constrained, r > r_constrained, and r_n > r
% Case I: Unconstrained
% compute the natural rate
% [Note: This is not the typical natural rate, since that would need r=r_n 
% at all future periods]
r_n = c_rn/(1+m);

if endog_r == 0
    % IA. Set interest rate exogenously
    r = r_exog;
else
    % IB. Use natural rate
    r = r_n;
    % IC. ZLB
    if r <= r_ZLB + 1e-9
        r = r_ZLB;
        ZLB = 1;
    end
end

% Government tax
Tau = g + r * B - tau_s * y_s - tau_b * y_b - P.xi * a_s - B_dot;

tax_s = tau_s * y_s + tax_share_s * Tau; 
tax_b = tau_b * y_b + tax_share_b * Tau;
posttax_inc_s = y_s - tax_s;
posttax_inc_b = y_b - tax_b;

% Individual debt
d_dot = r * (d+B) - B_dot + posttax_inc_s - c_s - P.xi*a_s + P.kappa_s * div;

% Case II: Constrained?
constrained = 0;
if exog_debt == 1
    rhs_constraint = exog_debt_d;
else
    rhs_constraint = kappa * p - P.kappa0;  % Constraint enters here! kappa * h_b - P.kappa0;
end

if d_dot/P.lambda + d > rhs_constraint
    % constraint is binding
    constrained = 1;
    % path of debt
    d_dot = P.lambda * (rhs_constraint - d);
    
    % compute natural rate
    % [Note: This is not the typical natural rate, since that would need r=r_n 
    % at all future periods]
    r_n = 1/(d + omega_b * B) * (c_s + omega_s*g + d_dot + omega_b*B_dot - ...
          (1 - tau_s*omega_b)*y_s - omega_s*tau_b*y_b - P.kappa_s*div + omega_b*P.xi*a_s);
    
    % compute interest rate
    if endog_r == 0
        % IIA. Set interest rate exogenously
        r = r_exog;
    else
        % IIB. Use natural rate
        r = r_n;
        % IIC. ZLB
        if r <= r_ZLB + 1e-9
            r = r_ZLB;
            ZLB = 1;
        end
    end
    
    % Government tax
    Tau = g + r * B - tau_s * y_s - tau_b * y_b - P.xi * a_s - B_dot;

    tax_s = tau_s * y_s + tax_share_s * Tau;
    tax_b = tau_b * y_b + tax_share_b * Tau;
    posttax_inc_s = y_s - tax_s;
    posttax_inc_b = y_b - tax_b;
    
end


% Euler for savers
c_s_dot = c_s * 1/P.inverse_eis * (r - P.rho_s - P.delta_s + P.delta_s * (c_s / P.rho_s / a_s)^P.inverse_eis * P.eta_s(a_s/Y_pot, P.a_thresh, P.eta0, P.eta1));  %%% TODO CHECK THIS

% Human capital
h_s_dot = r*h_s - posttax_inc_s;
h_b_dot = r*h_b - posttax_inc_b;

% Price of a tree
p_dot = r * p - Yhat;

% Physical capital k_dot = I - P.delta * k; % already determined above

% Investment
Q_dot = r*Q + c_Q; 

% Firm value
J_dot = r*J - pi + I + Zeta*k; 

% q 
q_dot = (r + P.nu)*q - 1;

a_s_dot = h_s_dot + d_dot + B_dot + J_dot;

I_dot = P.epsilon*Q_dot*k + c_I; 
Zeta_hat = Zeta_p*I_dot + c_Zeta; 

% Output     
if endog_r == 0 || ZLB == 1
    % write variables as linear transformation of Y_hat_dot; compute the slopes and intercepts 
    y_0_s = f_ns_k * k_dot * n - (w_s+f_ns_ns+f_ns_nb) * (MPK*k_dot + f_t) / (w_s+w_b);
    y_y_s = (w_s+f_ns_ns+f_ns_nb) / (w_s+w_b);
    y_0_b = f_nb_k * k_dot * n - (w_b+f_nb_nb+f_nb_ns) * (MPK*k_dot + f_t) / (w_s+w_b);
    y_y_b = (w_b+f_nb_nb+f_nb_ns) / (w_s+w_b);
    Tau_0 = g_dot + r_exog_dot * B + r * B_dot - tau_s_dot * y_s - tau_b_dot * y_b - P.xi * a_s_dot ...
            - tau_s * y_0_s - tau_b * y_0_b;
    Tau_y = - tau_s * y_y_s - tau_b * y_y_b;
    if exog_debt == 1
        rhs_constraint_dot = exog_debt_d_dot;
    else
        rhs_constraint_dot = kappa_dot*Q*k + kappa*p_dot;  
    end
    d_0 = P.lambda * (rhs_constraint_dot - d_dot);
    d_y = 0;
    omega_0_s = (1/(y_s+y_b)-y_s/(y_s+y_b)^2) * y_0_s;
    omega_y_s = (1/(y_s+y_b)-y_s/(y_s+y_b)^2) * y_y_s;
    c_0 = r_exog_dot*(B+d) + r*(B_dot+d_dot) - P.xi*a_s_dot - tau_s_dot*y_s ...
          + (1-tau_s)*y_0_s - Tau*omega_0_s - Tau_0 * omega_s - P.kappa_s * (y_0_s+y_0_b+I_dot+Zeta_hat);
    c_y = (1-tau_s)*y_y_s - Tau*omega_y_s - Tau_y * omega_s + P.kappa_s * (1 - y_y_s - y_y_b);
    Yhat_dot = (c_s_dot+B_ddot+d_0-c_0)/(c_y-d_y);
    
    n_dot = (Yhat_dot - f_k * k_dot - f_t) / f_n;
    
else
    Yhat_dot = f_k * k_dot + f_t;
    n_dot = 0;
end

% Borrower consumption
c_b_dot = Yhat_dot - c_s_dot - g_dot - I_dot - Zeta_hat;

% Put everything together
ydot = [c_s_dot, c_b_dot, d_dot, h_s_dot, h_b_dot, n_dot, kappa_dot, k_dot, J_dot, Q_dot, q_dot, p_dot]';
if any(isnan(ydot))
    warning('NaN encountered in odefun')
end

% extra output
extra.r = r; extra.ZLB = ZLB; extra.r_ZLB = r_ZLB;
extra.tau_s = tau_s; extra.tau_b = tau_b;
extra.w_s = w_s; extra.w_b = w_b;
extra.omega_s = omega_s; extra.omega_b = omega_b;
extra.tax_s = tax_s; extra.tax_b = tax_b; extra.Tau = Tau;
extra.y_s = y_s; extra.y_b = y_b;
extra.posttax_inc_s = posttax_inc_s; extra.posttax_inc_b = posttax_inc_b;
extra.g = g; extra.B = B;
extra.a_s = a_s; extra.a_b = a_b;
extra.r_n = r_n;
extra.d_dot = d_dot;
extra.endog_r = endog_r; 
extra.constrained = constrained;
extra.pledgable_wealth = rhs_constraint;
extra.pi = pi; extra.MPK = MPK; extra.I = I; extra.div = div;
extra.Y_pot = Y_pot; extra.n = n;
extra.Yhat = Yhat;
extra.d_tilde = d / q;

extra.check = c_s + d_dot + B_dot - r*(d+B) - (1-tau_s)*y_s + omega_s*Tau - P.kappa_s * div;

if abs(extra.check) > 1e-1
    1;
end
end



%%% BCFUN %%%
function res = bcfun(yL,yR,P,shock)
% this function imposes the boundary conditions

[y_dot, extra] = odefun(1e6, yR(:,end), P.endog_r_at_T, P, shock); % 1e6 = inf
[~, extra0] = odefun(0, yL(:,1), P.endog_r_at_0, P, shock); % 1e6 = inf

i=1;
c_s_inf = yR(i,end); c_s_dot_inf = y_dot(i);
i=i+1;
c_b_inf = yR(i,end);
i=i+1;
d0 = yL(i,1); d_inf = yR(i,end);  
i=i+1;
h_s_inf = yR(i,end); h_s_dot_inf = y_dot(i);
i=i+1;
h_b_dot_inf = y_dot(i);
i=i+1;
n0 = yL(i,1); n_inf = yR(i,end); 
i=i+1;
kappa_inf = yR(i,end);
i=i+1;
k_inf = yR(i,end); 
k0 = yL(i,1); 
i=i+1;
J_inf = yR(i,end); J_dot_inf = y_dot(i);
i=i+1;
Q_inf = yR(i,end); Q_dot_inf = y_dot(i);
i=i+1;
q0= yL(i,1); q_dot_inf = y_dot(i);
i=i+1;
p_dot_inf = y_dot(i);


%%% Aux objects
omega_s_inf = extra.omega_s;
y_s_inf = extra.y_s;
B_inf = extra.B;
a_s_inf = h_s_inf + d_inf + B_inf + J_inf;
I_inf = P.epsilon * (Q_inf-1) * k_inf + P.delta * k_inf;
Tau_inf = extra.Tau; 
tau_s_inf = P.tau_s(1e6);
g_inf = P.g(1e6);
r_inf = extra.r;
div_inf = extra.div;
r0 = extra0.r;
Yhat_inf = extra.Yhat;

% boundary conditions at t=0 or t=T
k=1; res = zeros(P.n_y * P.n_regions,1);
res(k) = d0 - P.d_tilde_0 * q0; k=k+1;  % initial debt
res(k) = c_s_dot_inf; k=k+1;
res(k) = h_s_dot_inf; k=k+1;
res(k) = h_b_dot_inf; k=k+1;
res(k) = Yhat_inf-c_b_inf-c_s_inf-g_inf-I_inf-P.Zeta((I_inf-P.delta*k_inf)/k_inf,P.epsilon)*k_inf; k=k+1;

res(k) = kappa_inf - P.kappa; k=k+1;
res(k) = k0 - P.kss; k=k+1;
res(k) = Q_dot_inf; k=k+1;
res(k) = J_dot_inf; k=k+1;
res(k) = q_dot_inf; k=k+1;
res(k) = p_dot_inf;k=k+1;

% see whether r is above ZLB and endogenous at t=0
if P.endog_r_at_0 == 1 && r0 >= extra0.r_ZLB + 1e-9
    % if so: fix n0 = 1
    res(k) = n0 - 1;k=k+1;
    % else consider Yhat at infinity
elseif P.endog_r_at_T == 0 || r_inf <= extra.r_ZLB + 1e-9
    res(k) = r_inf * (d_inf+B_inf) + (1-tau_s_inf)*y_s_inf - omega_s_inf*Tau_inf - P.xi * a_s_inf - c_s_inf + div_inf;k=k+1;
else
    res(k) = Yhat_inf - P.Yhat_inf;k=k+1;
end

% boundary conditions in the middle
for l = 1:P.n_regions-1
    for i = 1:P.n_y
        res(k) = yR(i,l) - yL(i,l+1);k=k+1;
    end
end

if any(isnan(res))
    warning('NaN encountered in bcfun')
end

end 