% This file computes the cts time model for Mian Straub Sufi

clear
close all
clc

%% (1a) Plot options 
width = '2.6in';
height = '2.5in';
% scale  = 0.65; % slides: 4x4 plot 0.45. slides 1x3 plot: 0.65 , Paper: 0.8
% suffix = '_slides'; % change to '_slides' for slides! '_paper' '_slides'
scale = 0.8;
suffix = '_paper';
globalstyle = 'very thick';
compile_pdf = 0;
write_tikz  = 0;

% Paths
H.pdflatexpath  = '/usr/local/texlive/2019/bin/x86_64-darwin'; % enter your own pdflatex path here

H.root_path = '';  % default: current folder
H.path_tikz = [H.root_path 'tikz/']; % folder for tikz files
H.compile_pdf_folder     = 'tikz_pdfs/'; %folder for pdf plots
H.compile_pdf_tikzfolder = 'tikz/'; % folder for tikz files within tikz_pdfs



%% (1b) Parameters 

% Set all parameters in P here

P.delta_s = 1/40;  % weight on bequest utility saver
P.delta_b = P.delta_s;  % weight on bequest utility borrower
P.rho_s = 0.1;  % discount rate saver
P.rho_b = 0.1;  % discount rate borrower
P.mu = 0.01;  % mass of savers (only important for the eta function)
P.inverse_eis = 1;  % inverse EIS
P.kappa_s = 0.04;
P.kappa_b = 1 - P.kappa_s;

P.kappa0 = 0;  % buffer stock
P.d0 = 0.45;  % initial debt
P.kappa = 0.027; %  r_ss *(P.d0 + P.kappa0);
P.kappa_dot = @(t) 0.;  
P.lambda = 0.5; %from RR shock response (1h)

P.a_thresh = 0.956208;
P.eta1 = 1.3245; % eta_tilde in paper
P.eta0 = 1; 

% Non-homothetic preferences
P.eta_s = @(a, a_thresh, eta0, eta1) 1 + eta0/eta1/a_thresh * (...
    (eta1*(a-a_thresh)<50) * log(1 + exp( (eta1*(a-a_thresh)<50) * eta1*(a-a_thresh) )) + ...
    (eta1*(a-a_thresh)>=50) * eta1*(a-a_thresh) ...
    );

P.eta_s_prime = @(a, a_thresh, eta0, eta1) eta0/a_thresh * (...
    (eta1*(a-a_thresh)<50) * exp( (eta1*(a-a_thresh)<50) * eta1*(a-a_thresh) )/(1 + exp( (eta1*(a-a_thresh)<50) * eta1*(a-a_thresh) )) + ...
    (eta1*(a-a_thresh)>=50) * eta1 ...
    );
P.eta_b = @(a, a_thresh, eta0, eta1, mu) P.eta_s(mu/(1-mu) * a, a_thresh, eta0, eta1);

% Government 
P.xi = 0;  % convenience yield
P.tau_s = @(t) 0;  % tax savers
P.tau_s_dot = @(t) 0.; 

P.tau_b = @(t) 0;  % tax borrowers
P.tau_b_dot = @(t) 0.; 
P.chi = 1.; % tax share on savers
P.phi = @(t) 0; % tax share shock
P.phi_dot = @(t) 0; 

P.g = @(t) 0.15;
P.B = @(t) 0.4;
P.g_dot = @(t) 0.;
P.B_dot = @(t) 0.;
P.B_ddot = @(t) 0.;

% Firm
P.delta = 0.06; % capital depreciation

P.epsilon = P.delta * 1; % adjustment cost. 1 = semi elasticity of I to Q
P.Zeta = @(x, epsilon) 1/(2 * epsilon) * x^2; % capital adjustment cost
P.Zeta_p = @(x, epsilon) x / epsilon;

P.alpha = 0.2; 
P.sigma = 1.0001; % capital low-skill labor elasticity of substitution
P.gamma = @(t) 0.04; % labor share of savers
P.gamma_dot = @(t) 0.; 
P.land_share = 0.05; % share of output going to land owners
P.kss = 1.7391; %1+2/3; % kss

% Production function
P.f = @(k,n_s,n_b,alpha,gamma,land_share,sigma,kss) (alpha/(1-gamma-land_share)*(k/kss)^((sigma-1)/sigma) + ...
                   (1-alpha-gamma-land_share)/(1-gamma-land_share)*n_b^((sigma-1)/sigma))^...
                   (sigma/(sigma-1)*(1-gamma-land_share)) * n_s^gamma;

P.Yhat0 = 1; 
P.Yhat_inf = 1; 

syms k n_s n_b alpha gamma land_share sigma kss % t
P.jacobian = matlabFunction(jacobian(P.f, [k n_s n_b alpha gamma land_share sigma kss]), 'Vars', [k n_s n_b alpha gamma land_share sigma kss]); 
P.hessian = matlabFunction(hessian(P.f, [k n_s n_b alpha gamma land_share sigma kss]), 'Vars', [k n_s n_b alpha gamma land_share sigma kss]); 

% durations of debt
D = 0.001; r_ss_duration = 0.06;
P.nu = 1/D - r_ss_duration;

% other
P.T_exog_to_endog = 0;  % at what time do we switch to endogenous r?
P.T_endog_to_exog = inf;  % at what time do we switch to exogenous r?
P.exog_debt = 0;  % use exogenous debt path?
P.r_exog = 0.06; % use in steady state
P.r_ZLB = -inf; % 0.035;

shock = struct(...  
    'r_exog', @(t) 0.06, ...
    'r_exog_dot', @(t) 0, ...
    'exog_debt', @(t) 0, ...
    'exog_debt_dot', @(t) 0, ...
    'r_ZLB', @(t) -inf ... % -inf ... Note: if ZLB binding permanently, better to use T_exog_to_endog for it!)
    );

 %%% Note: if a field in shock is set, overwrite corresponding field in P with current value in odefun
 %%% Also, if a field is set, precompute its time derivatives in compute_td
 %%% ahead of time.

P.abstol = 1e-9; P.reltol = 1e-9; P.NMax = 1000; T = 100; 

%% (1c) Common Structs 
Z_common = struct;
Z_common.fields_to_plot = {'d','debt_service_GDP','Yhat','r','check','k','omega_s', 'n', 'q'}; %'zeta'
Z_common.field_labels   = {'debt', 'debt service / GDP', 'Output', 'interest rate','check','k','labor inc share s', 'n', 'q'};
n_f = length(Z_common.fields_to_plot);
Z_common.stepsize       = zeros(1,n_f); % if zero, then Matlab's stepsize is used
Z_common.yaxis_percent  = ones(1,n_f);
Z_common.x_label        = 'years';
Z_common.y_label        = '';
Z_common.first_is_norm  = zeros(1,n_f); % use first tdlist as normalization. if = 2, first entry is not shown
Z_common.max_T          = min(25,T); 
Z_common.height         = height;
Z_common.width          = width;
Z_common.globalstyle    = globalstyle;
Z_common.compile_pdf    = compile_pdf;
Z_common.write_tikz     = write_tikz;
Z_common.scale          = scale;

P_common = P;


%% (1d) Calibration

%%% Ccompute steady state automatically afterwards and store
%%% calibration in P

d_target = 0.45;
MPC_target = 0.01;
state_level_target = 5; 
r_target = 0.055;
target = [d_target, MPC_target, state_level_target, r_target];


kappa_guess = P.kappa;
a_thresh_guess = P.a_thresh;
eta1_guess = P.eta1;
delta_s_guess = P.delta_s;
k_guess = P.kss;
init_guess= [kappa_guess, a_thresh_guess, eta1_guess, delta_s_guess, k_guess];


[kappa, a_thresh, eta1, delta_s, k_ss, ss] = calibrate_unified(P, shock, target, init_guess);  

disp(['k: ' num2str(k_ss,'%5.10f') ', kappa: ' num2str(kappa,'%5.10f') ', a_thresh: ' num2str(a_thresh,'%5.10f') ', eta1: ' num2str(eta1,'%5.10f') ', delta_s: ' num2str(delta_s,'%5.10f')])

P.kss = k_ss; P.a_thresh = a_thresh; P.eta1 = eta1; P.delta_s = delta_s; P.r_exog = ss.r;
P.kappa = kappa;

P_common = P;

% Compute steady state

[ss_vec, ss] = steadystate_unified(P);

r_ss = ss.r; P.d0 = ss.d; P.d_tilde_0 = P.d0/ss.q; P.q0 = ss.q; P.kss = ss.k;

disp(['After tax non-debt income of savers: ' num2str(ss.a_s*ss.r - ss.r*ss.d)])

%%% Set common structs to avoid overwriting
shock_common = shock;
P_common = P;
P_common_hom = P;
P_common_hom.delta_s = 0; P_common_hom.delta_b = 0; P_common_hom.rho_s = r_target;

return

%% (1e) Steady state with greater inequality

P = P_common;
shock = shock_common; 

P.gamma = @(t) 0.04 + 0.07;   % labor share of savers
P.sigma = 1.001; 

[ss_vec, ss] = steadystate_unified(P);

% Show after tax non-priv-debt income of savers: [This is what should be used in
% simple model, without public debt]
disp(['After tax non-debt income of savers: ' num2str(ss.a_s*ss.r - ss.r*ss.d)])
disp(['Private wealth in new steady state: ' num2str(ss.J + ss.B)])
return




%% (1f) Simple model's steady state
clearvars tdlist
P = P_common; shock = shock_common;

P.alpha = 1e-7; 
P.sigma = 1.0001; % capital low-skill labor elasticity of substitution
P.gamma = @(t) 0.06; % labor share of savers  <--- enter "after tax non-debt income of savers" here
P.gamma_dot = @(t) 0.;
P.land_share = 0.0; % share of output going to land owners
P.B = @(B) 0.;
P.g = @(t) 0; shock.g = @(t) 0;


d_target = 0.45;
MPC_target = 0.01; 
state_level_target = 5.22;
r_target = 0.055;
target = [d_target, MPC_target, state_level_target, r_target];

kappa_guess = P.kappa;
a_thresh_guess = P.a_thresh;
eta1_guess = P.eta1;
delta_s_guess = P.delta_s;
k_guess = P.alpha*P.kss;

init_guess= [kappa_guess, a_thresh_guess, eta1_guess, delta_s_guess, k_guess];

[kappa, a_thresh, eta1, delta_s, k_ss, ss] = calibrate_unified(P, shock, target, init_guess);  

disp(['k: ' num2str(ss.k,'%5.10f') ', kappa: ' num2str(kappa,'%5.10f') ', a_thresh: ' num2str(a_thresh,'%5.10f') ', eta1: ' num2str(eta1,'%5.10f') ', delta_s: ' num2str(delta_s,'%5.10f')])

P.kss = k_ss; P.a_thresh = a_thresh; P.eta1 = eta1; P.delta_s = delta_s; P.r_exog = ss.r;
P.kappa = kappa;
[ss_vec, ss] = steadystate_unified(P);

P.d_tilde_0 = ss.d/ss.q; P.d0 = ss.d;


P_simple = P;
P_simple_hom = P; P_simple_hom.delta_s = 0; P_simple_hom.delta_b = 0;
P_simple_hom.rho_s = ss.r;
shock_simple = shock;

return

%% (1g) Steady state with greater inequality in simple model

P = P_simple;
shock = shock_simple; 

P.gamma = @(t) 0.10;  % labor share of savers

[ss_vec, ss] = steadystate_unified(P);

disp(['After tax non-debt income of savers: ' num2str(ss.a_s*ss.r - ss.r*ss.d)])

return


%% (1h) MP shock with nat rate afterwards (RR shock for calibration)
% 
clearvars tdlist
k = 1;
P = P_simple; shock = shock_simple; 
[~, ss] = steadystate_unified(P);

% set MP shock
step = @(x) (tanh(-x)+1)/2;
step_prime = @(x) -1/2 * (1 - tanh(-x).^2);

%%% step function
kappa0 = 0;
delta_t = 0.038;
Delta = -0.005; kink = 10; t0 = 4+delta_t;  % parameters
shock.r_exog = @(t) Delta*step((t-t0)*kink) + ss.r;
shock.r_exog_dot = @(t) Delta*step_prime((t-t0)*kink) * kink;

% non-hom
P.kappa0 = kappa0;  % buffer stock
P.kappa = 0.04*(0.80+P.kappa0)/ss.omega_b; % pledgable fraction

tdlist{k} = compute_td_unified(P,T,shock,0,7);k=k+1;

% add constant steady state lines
tdlist{k}.t = tdlist{k-1}.t;
tdlist{k}.d = P.d0 + 0*tdlist{k}.t;
tdlist{k}.r = ss.r + 0*tdlist{k}.t;

Z = Z_common; Z.max_T = 10;
Z.plot_name      = 'mp_romerromer';
Z.fields_to_plot = {'r','d'};
Z.field_labels   = {'Interest rate','Household debt / GDP'};
% Z.yaxis          = {{3.725, 4.025}};
Z.xticks         = {0,5,10};
Z.colorlist      = {'black','gray'};
Z.globalstyle    = '';
Z.stylelist      = {'solid, very thick','dotted, thick'};
Z.width          = '3in';
Z.legend_        = {'MP shock'};
Z.plot_name = [Z.plot_name suffix];
Z.compile_pdf = 0; Z.write_tikz = 0;
plot_this(Z,tdlist,H);

return


%% (2a) Replicate the simple model's inequality transition here
%  Section 5.1 Inequality: Figures 7 + 8

clearvars tdlist
shock = shock_simple;  P.NMax = 4000;

gamma = 0.06;
Delta = 0.04;
up_step = @(x) (tanh(x)+1)/2;
up_step_prime = @(x) 1/2 * (1 - tanh(x).^2); t0 = 20; kink = 0.1;
gamma_fn = @(t) gamma + Delta*up_step((t-t0)*kink);
gamma_fn_dot = @(t) Delta*up_step_prime((t-t0)*kink) * kink;

% Homothetic model
P = P_simple_hom;
P.gamma = gamma_fn;
P.gamma_dot = gamma_fn_dot;
j=1;
tdlist{j} = compute_td_unified(P,T,shock,0,9); j=j+1;

% non-homothetic model
P = P_simple;
P.gamma = gamma_fn;
P.gamma_dot = gamma_fn_dot;
tdlist{j} = compute_td_unified(P,T,shock,0,9); j=j+1;

% gray line
P = P_simple;
tdlist{j} = compute_td_unified(P,T,shock,0,0); j=j+1;

%% plot inequality transition (Figure 7)
Z = Z_common; Z.max_T = 50;
Z.plot_name      = 'incineq_noZLB';
Z.fields_to_plot = {'omega_s','r','d','debt_service_GDP'}; 
Z.field_labels   = {'Top 1\\% income share','Interest rate','Household debt / GDP','Debt service / GDP'};
Z.globalstyle    = '';
Z.yaxis          = {{6,10},{1.75,8.5},{45,115},{1.5,4.5}};
Z.width          = '3in';
Z.colorlist      = {'blue','black','gray'};
Z.stylelist      = {'dashed,very thick','solid, very thick','dotted, thick'};
% Z.width          = '3in';
Z.legend_        = {'Homothetic model', 'Non-homothetic model '};
Z.plot_name = [Z.plot_name suffix];
if strcmpi(suffix,'_slides'), Z.scale = 0.45; end
plot_this(Z,tdlist,H);


% plot net worth of borrowers (Figure 8)
tdlist_nw = {};
tdlist_nw{1} = tdlist{2}; % this will actually have net worth
tdlist_nw{2} = tdlist_nw{1}; tdlist_nw{2}.net_worth_b = tdlist_nw{1}.pledgable_wealth - tdlist{3}.pledgable_wealth(1);
tdlist_nw{3} = tdlist_nw{1}; tdlist_nw{3}.net_worth_b = tdlist{3}.d(1) - tdlist_nw{1}.d;
tdlist_nw{4} = tdlist{3};

Z = Z_common; Z.max_T = 50;
Z.plot_name      = 'incineq_borrower';

Z.fields_to_plot = {'net_worth_b'}; %'zeta'
Z.field_labels   = {''};

Z.globalstyle    = '';
Z.y_label        = 'change relative to s.s. output';
Z.yaxis          = {{-65,65}};
Z.yticks_list    = {{-60,-40,-20,0,20,40,60}};
Z.width          = '3in';
Z.colorlist      = {'black','blue','red','gray'};
Z.stylelist      = {'solid, very thick','loosely dashed,very thick','dashed, very thick', 'dotted, thick'};
% Z.width          = '3in';
Z.legend_        = {'Net worth', 'Pledgable wealth', 'Debt'};
Z.plot_name = [Z.plot_name suffix];
plot_this(Z,tdlist_nw,H);

return


%% (2b) Deficit spending (no ZLB)
% Section 6.1 Implications for Fiscal Policy: Figure 11

clearvars tdlist
k=1; shock = shock_simple;
P = P_simple;
[~, ss] = steadystate_unified(P);

%%% exp declining g policy
half_life = 2; alpha = -log(0.5)/half_life; Delta = 0.055; t0 = 4;
P.g = @(t) Delta*exp(-alpha*t);
P.g_dot = @(t) -alpha*Delta*exp(-alpha*t);

Delta = 0.1; half_life = 4; alpha = -log(0.5)/half_life;
P.B = @(t) Delta - Delta*exp(-alpha*t);
P.B_dot = @(t) alpha*Delta*exp(-alpha*t);
P.B_ddot = @(t) alpha^2 * Delta*exp(-alpha*t);

tdlist{k} = compute_td_unified(P,T,shock,0,7);k=k+1;

%%% Homothetic model
P = P_simple_hom;
[~, ss] = steadystate_unified(P);

%%% exp declining g policy
half_life = 2; alpha = -log(0.5)/half_life; Delta = 0.055; t0 = 4;
P.g = @(t) Delta*exp(-alpha*t);
P.g_dot = @(t) -alpha*Delta*exp(-alpha*t);

Delta = 0.1; half_life = 4; alpha = -log(0.5)/half_life;
P.B = @(t) Delta - Delta*exp(-alpha*t);
P.B_dot = @(t) alpha*Delta*exp(-alpha*t);
P.B_ddot = @(t) alpha^2 * Delta*exp(-alpha*t);

tdlist{k} = compute_td_unified(P,T,shock,0,7);k=k+1;

% add constant steady state lines
tdlist{k}.t = tdlist{k-1}.t;
tdlist{k}.d = P.d0 + 0*tdlist{k}.t;
tdlist{k}.debt_service_GDP = ss.r*P.d0 + 0*tdlist{k}.t;
tdlist{k}.r = ss.r + 0*tdlist{k}.t;
tdlist{k}.B = 0 + 0*tdlist{k}.t;
k = k+1;

% plot deficit spending shock (Figure 11)
Z = Z_common; Z.max_T = 30;
Z.fields_to_plot = {'B','r','d'};
Z.field_labels   = {'Gov. debt / GDP', 'Interest rate','Household debt / GDP'};
Z.plot_name      = 'def_spending_noZLB';
Z.colorlist      = {'black','blue','gray'};
Z.yaxis          = {{-0.5, 10},{5.0,6.7},{42,48}};
Z.globalstyle    = '';
Z.stylelist      = {'solid, very thick','dashed, very thick','dotted, thick'};
Z.legend_        = {'Non-homothetic model','Homothetic model'};
Z.plot_name = [Z.plot_name suffix];
plot_this(Z,tdlist,H);

return



%% (2c) Falling into the liquidity trap in response to greater inequality
% Section 7.1 The Debt Trap Steady State: Figure 13
clearvars tdlist

P = P_simple; shock = shock_simple;  P.NMax = 4000;

P.abstol = 1e-5; P.reltol = 1e-5; % to speed things up
gamma = 0.06+0.04;
Delta = 0.01;
up_step = @(x) (tanh(x)+1)/2;
up_step_prime = @(x) 1/2 * (1 - tanh(x).^2); t0 = 20; kink = 0.1;
gamma_fn = @(t) gamma + Delta*up_step((t-t0)*kink);
gamma_fn_dot = @(t) Delta*up_step_prime((t-t0)*kink) * kink;


P.gamma = gamma_fn;
P.gamma_dot = gamma_fn_dot;
[ss_vec , ss] = steadystate_unified(P);
P.d0 = ss.d; P.d_tilde_0 = ss.d/ss.q; 
P.kappa = ss.r * (P.d0+P.kappa0);

% no ZLB
k=1;
tdlist{k} = compute_td_unified(P,T,shock,0,9);k=k+1;

% add ZLB?
P.r_ZLB = 0.02;
shock.r_ZLB = @(t) P.r_ZLB;
shock.r_exog = @(t) P.r_ZLB;
% P.T_exog_to_endog = 10;

tdlist{k} = compute_td_unified(P,T,shock,0,9);k=k+1;

% add dashed lines
tdlist{k}.t = tdlist{k-1}.t;
tdlist{k}.omega_s = P.gamma(0) + 0*tdlist{k}.t;
tdlist{k}.d = P.d0 + 0*tdlist{k}.t;
tdlist{k}.r = r_ss + 0*tdlist{k}.t;
tdlist{k}.debt_service_GDP = r_ss*P.d0 + 0*tdlist{k}.t;
tdlist{k}.Ygap = 0*tdlist{k}.t;
k = k+1;

% plot debt trap transition (Figure 13)
Z = Z_common; Z.max_T = 50;
Z.plot_name      = 'incineq_ZLB';
Z.fields_to_plot = {'d','r','Ygap'};
Z.field_labels   = {'Household debt / GDP','Interest rate','Output gap'};
Z.yaxis          = {{100, 165},{1.5,5},{-1.7,0.2}}; 
Z.globalstyle    = '';
Z.colorlist      = {'blue','black','gray'};
Z.stylelist      = {'dashed,very thick','solid,very thick','dotted, thick'};
Z.width          = '2.55in';
Z.legend_        = {'Without ZLB', 'ZLB at $r=2\\%$'};
Z.plot_name = [Z.plot_name suffix];
plot_this(Z,tdlist,H);


return




%% (3) Rising income inequality Q
% Section 9 Quantative Exploration: Figure 15
clearvars tdlist
T = 150;

sigma_list = [P_common.sigma*ones(1,4) 2 2 0.5 P_common.sigma];
epsilon_list = [P_common.epsilon*ones(1,7) 0.00001];
homothetic = [1 1 zeros(1,6)];
gamma = 0.04;
Delta_list = [0.03 0.07 0.03 0.07 0.03 0.07* ones(1,4)];
use_previous = [0 1 0 3 3 5 3*ones(1,3)];
kill_tdlist = [1 0 1 0 1 zeros(1,4)];
add_non_perfect_foresight = 1;
previousrun = 0;

ss_list = {};

J = length(sigma_list);

for j = 1:J
    if homothetic(j)
        P = P_common_hom;
    else
        P = P_common;
    end
    P.abstol = 1e-5; P.reltol = 1e-5; % to speed things up
    shock = shock_common;
    
    % Change duration of debt
    [ss_vec, ss] = steadystate_unified(P);
    P.d_tilde_0 = ss.d/ss.q; ss_list{j} = ss_vec;
    
    % Write step function
    up_step = @(x) (tanh(x)+1)/2; t0 = 20; kink = 0.1;
    Delta = Delta_list(j); 
    up_step_prime = @(x) 1/2 * (1 - tanh(x).^2); t0 = 20; kink = 0.1;
    gamma_fn = @(t) gamma + Delta*up_step((t-t0)*kink);
    gamma_fn_dot = @(t) Delta*up_step_prime((t-t0)*kink) * kink;
    P.gamma = gamma_fn;
    P.gamma_dot = gamma_fn_dot;
    P.sigma = sigma_list(j);
    
    % adj cost
    P.epsilon = epsilon_list(j);
    
    % run
    if use_previous(j) > 0
        % construct previous guess by rescaling
        jp = use_previous(j);
        sol_guess = tdlist{jp}.sol;
        sol_guess.y = ss_list{j} + (sol_guess.y - ss_list{jp})/Delta_list(jp) * Delta_list(j);
        sol_guess.yp = sol_guess.yp/Delta_list(jp) * Delta_list(j);
        
        tdlist{j} = compute_td_unified(P,T,shock,sol_guess,0);
    else
        tdlist{j} = compute_td_unified(P,T,shock,0,9);
    end
end


%%% Add non-perfect foresight?
if add_non_perfect_foresight
    
    P = P_common;
    P.abstol = 1e-4; P.reltol = 1e-4; 
    smooth_path = 1; smoothing_periods = 100;
    
    %%% Set true inequality function
    up_step = @(x) (tanh(x)+1)/2; t0 = 20; kink = 0.1;
    Delta = 0.07; 
    up_step_prime = @(x) 1/2 * (1 - tanh(x).^2); t0 = 20; kink = 0.1;
    
    gamma_fn_correct = @(t) gamma + Delta*up_step((t-t0)*kink);
    gamma_fn_correct_dot = @(t) 0; 
    
    % Set dates where shock surprise happens
    surprise_times = [0:0.25:60 inf]; 
    N = length(surprise_times)-1;
    
    % construct gamma functions that correspond to this
    gamma_fn_list = {};
    for i = 1:N
        t0 = surprise_times(i);
        t1 = surprise_times(i+1);
        gamma_fn_list{i} = @(t) gamma_fn_correct(t0);  
    end
    

    %%% run through all shocks
    % general options
    T = 150; 
    % initialize
    d_tilde_0 = P.d_tilde_0;
    previousrun = 0;

    % loop
    for i = 1:N
        disp([num2str(i) ' out of ' num2str(N)])
        gamma_fn = gamma_fn_list{i};
        P.gamma = gamma_fn;
%         disp(P)

        P.d_tilde_0 = d_tilde_0;

        tdlist_singleshock{i} = compute_td_unified(P,T,shock,previousrun,5);

        % update initial conditions
        if i < N
            t0 = surprise_times(i);
            t1 = surprise_times(i+1);
            k0 = interp1(tdlist_singleshock{i}.t, tdlist_singleshock{i}.k, t1-t0);
            d_tilde_0 = interp1(tdlist_singleshock{i}.t, tdlist_singleshock{i}.d_tilde, t1-t0);
            interval = (tdlist_singleshock{i}.t>=t0);
        end
    end

    %%% Put the transitions together
    fields = fieldnames(tdlist_singleshock{1});
    for i_field = 1:length(fields)
        field = fields{i_field};
        path.(field) = [];
        for i = 1:N
            t0 = surprise_times(i);
            t1 = surprise_times(i+1);
            interval = (tdlist_singleshock{i}.t <= t1 - t0);
            if ~strcmp(field,'t') && ~strcmp(field,'ss') && ~strcmp(field,'sol')
                path.(field) = [path.(field), tdlist_singleshock{i}.(field)(interval)];
            elseif strcmp(field,'t')
                path.t = [path.t, t0 + tdlist_singleshock{i}.t(interval)];
            end
        end
               
        if isempty(j)
            j=1;
        else
            j = length(sigma_list)+1;
        end
        if smooth_path
            % smooth
            if ~strcmp(field,'ss') && ~strcmp(field,'sol')
                tdlist{j}.(field) = movmean(path.(field), smoothing_periods);
            end
        else
            if ~strcmp(field,'ss') && ~strcmp(field,'sol')
                tdlist{j}.(field) = path.(field);
            end
        end
    end
  
tdlist{j}.t = tdlist{j}.t -tdlist{j}.t(1);
    
end

% kill guesses
selec = ~kill_tdlist==1;
selec = selec(1:J);
if add_non_perfect_foresight
    selec = [selec 1];
end
tdlist = {tdlist{selec==1}};

%%% gray lines
tdlist{length(tdlist)+1} = compute_td_unified(P_common,T,shock,0,0);

% plot inequality transitions (Figure 15)
Z = Z_common; Z.max_T = 55;
Z.plot_name      = 'incineq_noZLB_Q';
Z.colorlist      = {'black!70','black','blue','blue','green','red','gray'};
Z.stylelist      = {'loosely dashed','solid','solid','dashed','dash dot','densely dashed','loosely dotted'};
Z.legend_        = {'Homothetic','Baseline','$\\sigma = 2$','$\\sigma = 0.5$','$K=const$','No perfect foresight'};
Z.fields_to_plot = {'r','d'};
Z.field_labels   = {'interest rate','debt'};
n_f = length(Z.fields_to_plot);
Z.yaxis          = {{2,9},{35,140}};
Z.stepsize       = zeros(1,n_f); % if zero, then Matlab's stepsize is used
Z.yaxis_percent  = ones(1,n_f);
Z.x_label        = 'years';
Z.y_label        = '';
Z.first_is_norm  = zeros(1,n_f); % use first tdlist as normalization. if = 2, first entry is not shown
Z.plot_name = [Z.plot_name suffix];
Z.width          = '4in';
plot_this(Z,tdlist,H);

return
